var CustomType = function CustomType () {};

var Any = /*@__PURE__*/(function (CustomType) {
  function Any () {
    CustomType.apply(this, arguments);
  }

  if ( CustomType ) Any.__proto__ = CustomType;
  Any.prototype = Object.create( CustomType && CustomType.prototype );
  Any.prototype.constructor = Any;

  Any.prototype.parse = function parse () {
    return true;
  };

  return Any;
}(CustomType));

/**
 * @param {any} value
 * @param {any} type
 * @param {string[]} path
 * @param {boolean} silent
 * @returns {any} value
 */
function typeCheck(value, type, path, silent) {
  if ( path === void 0 ) path = [];
  if ( silent === void 0 ) silent = false;

  if (type && type.prototype instanceof CustomType) {
    var ref = type();
    var parse = ref.parse;
    if (typeof parse === 'function') { return parse(value); }
  }

  var retype = ((typeof type === 'function') ? type() : type);
  var typed = typeof retype;

  if (type && type.constructor === RegExp) {
    var typeChecked = retype.test(value);
    if (!typeChecked && !silent) {
      console.error(("Type error: Expected " + (path.length > 0 ? ("`" + (path.join('.')) + "`") : 'value') + " as `" + value + "` to pass regexp `" + retype + "`"));
    }
    return typeChecked;
  }

  if (value && value.constructor === type) {
    return true;
  }

  if (typed === 'object' && typeof value === 'object') {
    var valid = true;

    if (type.length === 1) {
      // eslint-disable-next-line no-restricted-syntax
      for (var item in value) {
        if ({}.hasOwnProperty.call(value, item)) {
          var typeChecked$1 = typeCheck(value[item], type[0], path.concat(item), silent);
          if (valid) {
            valid = typeChecked$1;
          }
        }
      }
    } else {
      var newValue = Object.assign({}, type, value);
      // eslint-disable-next-line no-restricted-syntax
      for (var item$1 in newValue) {
        if ({}.hasOwnProperty.call(newValue, item$1)) {
          var typeChecked$2 = typeCheck(value[item$1], type[item$1], path.concat(item$1), silent);
          if (valid) {
            valid = typeChecked$2;
          }
        }
      }
    }

    return valid;
  }

  // eslint-disable-next-line valid-typeof
  var checked = typeof value === typed;

  if (!checked) {
    if (value === undefined) {
      if (!silent) {
        console.error(("Type error: " + (path.length > 0 ? ("`" + (path.join('.')) + "`") : 'Value') + " is undefined! Required value of type `" + typed + "`"));
      }
    } else
    if (retype === undefined) {
      if (!silent) {
        console.error(("Type error: " + (path.length > 0 ? ("`" + (path.join('.')) + "`") : 'Value') + " is defined as `" + value + "`! But it should not be defined at all!"));
      }
    } else
    if (!silent) {
      console.error(("Type error: `" + value + "` is not of type `" + typed + "`" + (path.length > 0 ? (" in `" + (path.join('.')) + "`") : '')));
    }
  }

  return checked;
}

/**
 * @param {any} value
 * @param {any[]} typeList
 * @returns {any} value
 */
function assert(value) {
  var typeList = [], len = arguments.length - 1;
  while ( len-- > 0 ) typeList[ len ] = arguments[ len + 1 ];

  if (typeList && typeList.length === 1) {
    if (!typeCheck(value, typeList[0])) {
      throw new TypeError('Type checker found some type mismatches!');
    }
  } else
  if (typeList && typeList.length > 1) {
    var valid = false;
    typeList.forEach(function (type) { return !valid && (valid = typeCheck(value, type, [], true)); });
    if (!valid) {
      console.error(("Type error: `" + value + "` is not any of types: " + (typeList.map(function (t) { return ("`" + (t.name) + "`"); }).join(', '))));
    }
  }

  return value;
}

/**
 * @param {any[]} types
 * @returns {typeof OrType}
 */
function Or() {
  var types = [], len = arguments.length;
  while ( len-- ) types[ len ] = arguments[ len ];

  var OrType = /*@__PURE__*/(function (CustomType) {
    function OrType () {
      CustomType.apply(this, arguments);
    }

    if ( CustomType ) OrType.__proto__ = CustomType;
    OrType.prototype = Object.create( CustomType && CustomType.prototype );
    OrType.prototype.constructor = OrType;

    OrType.prototype.parse = function parse (value) {
      return assert.apply(void 0, [ value ].concat( types ));
    };

    return OrType;
  }(CustomType));

  return OrType;
}

var types = {
  Any: Any,
  Or: Or,
  Custom: CustomType,
};

/**
 * @param {any} value
 * @param {any[]} typeList
 * @returns {boolean}
 */
function check(value) {
  var typeList = [], len = arguments.length - 1;
  while ( len-- > 0 ) typeList[ len ] = arguments[ len + 1 ];

  try {
    assert.apply(void 0, [ value ].concat( typeList ));
  } catch (e) {
    return false;
  }

  return true;
}

export { assert, check, types };
//# sourceMappingURL=letype.es.js.map
